#pragma once

#include <Colors.hpp>
#include <J3ML/LinearAlgebra/Vector2.hpp>
#include <J3ML/LinearAlgebra/Matrix4x4.hpp>

namespace JGL {
    struct Instance2D;
    struct Instance3D;
}

/// Information provided to JGL for each instance of an object you're rendering in a batch.
/// @note This is non-negotiable. This information is interleaved in *the same buffer* in v-ram for performance reasons - Redacted.
struct JGL::Instance2D {
public:
    Color4 color;
    Vector2 position;
    Vector2 size;
    Vector2 scale;
    float rotation;
public:
    /// @param position The position of the instance in world space.
    /// @param size The size of the instance.
    /// @param scale A multiplier to be applied to the size.
    /// @param rotation The rotation in radians.
    /// @param color  A 3-or-4 channel color value. @see class Color3, class Color4.
    /// @note If the thing you're drawing is textured you probably want Colors::White
    Instance2D(const Color4& color, const Vector2& position, const Vector2& size, const Vector2& scale = Vector2::One, float rotation = 0.0f) :
    color(color), position(position), size(size), scale(scale), rotation(rotation) {};
    Instance2D() = default;
    ~Instance2D() = default;
};

struct JGL::Instance3D {
public:
    Matrix4x4 instance_matrix;
    Color4 color;
public:
    /// @param instance_matrix A matrix containing rotation matrix, position, and scale.
    /// @param color  A 3-or-4 channel color value. @see class Color3, class Color4.
    /// @note If the thing you're drawing is textured you probably want Colors::White
    Instance3D(const Matrix4x4& instance_matrix, const Color4& color) : instance_matrix(instance_matrix), color(color) {};
    Instance3D() = default;
    ~Instance3D() = default;
};




