#pragma once

#include <JGL/types/Texture.h>

namespace JGL {
    struct AtlasRegion {
        Vector2i position = { 0, 0 };
        Vector2i size = { 0, 0 };
    };
    class TextureAtlas;
}

// A texture which actually contains multiple such that you can pass it in to draw sprite or draw partial sprite.
// It also makes for more efficient sprite batch because you don't have to swap textures between.
class JGL::TextureAtlas : public Texture {
protected:
    std::vector<AtlasRegion> regions;
public:
    unsigned int RegionCount() { return regions.size(); }
    [[nodiscard]] AtlasRegion GetRegion(unsigned int region) const;
public:
    /// Create a texture atlas from multiple separate images.
    /// @param pixels The array(s) of pixels.
    /// @param sizes The size of each image.
    /// @param texture_count The number of textures this atlas will contain.
    TextureAtlas(const Color4** pixels, const Vector2i** sizes, unsigned int texture_count, FilteringMode filtering_mode = FilteringMode::NEAREST);

    /// Create a texture atlas from a single image
    /// @param pixels The array of pixels.
    /// @param size The size of the image.
    /// @param regions The individual regions that will make up your atlas.
    /// @param region_count The number of regions there are.
    TextureAtlas(const Color4* pixels, const Vector2i& size, AtlasRegion** regions, unsigned int region_count, FilteringMode filtering_mode = FilteringMode::NEAREST, SampleRate anisotropy = SampleRate::NONE);
    ~TextureAtlas() = default;
};