#include <JGL/types/Material.h>
#include <J3ML/LinearAlgebra/Vector4.hpp>

JGL::Material::Material(const Color4& ambient_reflection, const Color4& diffuse_reflection, const Color4& specular_reflection, const Color4& light_emission, float& shine) {
    ambient_ref = ambient_reflection;
    diffuse_ref = diffuse_reflection;
    specular_ref = specular_reflection;
    emission = light_emission;
    shininess = shine;
}

void JGL::Material::SetActiveMaterial(const Material& material) {
    Vector4 ambient = { material.ambient_ref.RN(), material.ambient_ref.BN(), material.ambient_ref.GN(), material.ambient_ref.AN() };
    Vector4 diffuse = { material.diffuse_ref.RN(), material.diffuse_ref.BN(), material.diffuse_ref.GN(), material.diffuse_ref.AN() };
    Vector4 specular = { material.specular_ref.RN(), material.specular_ref.BN(), material.specular_ref.GN(), material.specular_ref.AN() };
    Vector4 emission = { material.emission.RN(), material.emission.BN(), material.emission.GN(), material.emission.AN() };

    // Determine which faces we're going to apply this material to.
    GLenum face = GL_FRONT;
    if (glIsEnabled(GL_CULL_FACE)) {
        GLint current; glGetIntegerv(GL_CULL_FACE_MODE, &current);
        if (current == GL_FRONT)
            face = GL_BACK;
    }

    glMaterialfv(face, GL_AMBIENT, ambient.ptr());
    glMaterialfv(face, GL_DIFFUSE, diffuse.ptr());
    glMaterialfv(face, GL_SPECULAR, specular.ptr());
    glMaterialfv(face, GL_EMISSION, emission.ptr());
    glMaterialf(face, GL_SHININESS, material.shininess);
}
